#!/usr/bin/python
# coding: utf-8

__author__ = "Emilio Ramon Garcia Ladona"
__maintainer__ = "Roger Samsó"
__status__ = "Development"


"""
This tool allows plotting the model simulation results. When running run.py with
the option -p, it shows up at the end of the simulation and displays the simulation
results. It can also run independently, in which case the results must be imported
from a csv file. In both cases, only two curves can be represetnted at the same
time.
"""

import matplotlib
matplotlib.use("TkAgg")
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
from matplotlib.figure import Figure
from tkinter.filedialog import askopenfilename, asksaveasfilename, askdirectory
from tkinter import messagebox, Button
import numpy as np
import os
from configparser import ConfigParser
import re

import pandas as pd
import tkinter as tk
import sys


class Plot_tool(tk.Frame):

    last_col = ''
    
    def __init__(self, master=None, data=None, results_folder=None, scenario=''):
        tk.Frame.__init__(self, master)  # esto define self.master
        self.master = master
        self.master.option_add('*tearOff', 'FALSE')  # menus no detaching
        self.master.title("MEDEAS Plotting tool")
        self.results_folder = results_folder
        self.column = ''

        # setting the default folder to store saved plots
        matplotlib.rcParams['savefig.directory'] = self.results_folder


        self.subplot = None     # axis canvas
        self.canvas = None  # canvas

        #  Data
        if data is not None:
            self.py_dict = data.to_dict(orient='list')
            self.time_updated = data.index.values
            self.scenario = scenario.capitalize()
        else:
            self.py_dict = {}
            self.time_updated = []
            self.scenario = scenario

        # second pair of data to plot
        self.py_dict2 = {}
        self.time_updated2 = []
        self.scenario2 = ''

        # generate funcname: (unit, legend) dictionary from file
        self.legend_by_name = {}
        try:
            with open(os.path.join('plotting', 'legend_by_name.txt'), 'r', encoding='utf-8') as f:
                legend_by_name_list = f.readlines()

        except:
            print('legend_by_name.txt cannot be read correctly\n No legends and units generated')
            #raise
        else:
            for line in legend_by_name_list:
                funcname, unit, legend = [x.strip() for x in line.split('@@')]
                self.legend_by_name[funcname] = (unit, legend)
        
        self.init_window()

    def init_window(self):
                
        self.pack(fill=tk.BOTH, expand=1)
        
        # Variables
        #logo = "medeas.png"
        #img = ImageTk.PhotoImage(file=logo)

        default_list = list(sorted(['tpe_from_res_ej', # total primary energy supply from RES (MToe/Year)
                        'total_extraction_nre_ej',  # Annual total extraction of non-renewable energy resources (EJ/Year)
                        'percent_res_vs_tpes',  # Percent of primary energy from RES in the TPES (%)
                        'temperature_change',  # Temperature of the Atmosphere and Upper Ocean, relative to preindustrial reference period (degreesC)
                        'total_land_requirements_renew_mha',  # Land required for RES power plants and total bioenergy (land competition + marginal lands (MHa)
                        'share_blue_water_use_vs_ar',  # Share of blue water used vs accessible runoff water (Dmnl)
                        'gdppc',  # GDP per capita (1995T$ per capita) ($/people)
                        'eroist_system',  # EROI standard of the system (Dmnl)
                        'tfes_intensity_ej_tdollar',  # Total final energy intensity (EJ/T$)
                        'real_tfec',  # Real total final energy consumption (EJ)
                        'gdp',  # Global GDP in T1995T$ (T$)
                        'population' # Population projection (people)
                        ]))

        with open(os.path.join(self.results_folder, 'last_output_conf.txt'), encoding='utf-8') as f:
            txt = f.read()
            d_keys = sorted([line.strip() for line in txt.split(';') if line.strip() not in default_list])

        # FIN Variables

        menubar = tk.Menu(self.master)
        self.master.config(menu=menubar)
        
        filemenu = tk.Menu(menubar)
        filemenu.add_command(label="Load data", command=self.open_file)
        filemenu.add_command(label="Exit", command=lambda: on_closing(self))
        menubar.add_cascade(label="File", menu=filemenu)

        p = tk.PanedWindow(self, orient=tk.HORIZONTAL)
        p.pack(side="left", fill=tk.BOTH, expand=1)

        f1 = tk.LabelFrame(p, text='Variables', width=100, height=100)
        f2 = tk.Frame(p, width=100, height=100)

        p.add(f1)
        p.add(f2)

        f3 = tk.Frame(f1)
        f3.pack(side=tk.TOP)

        all_vars = default_list + ['--------'] + d_keys

        right_scrollbar = tk.Scrollbar(f1, orient=tk.VERTICAL)
        right_scrollbar.pack(side=tk.RIGHT, fill=tk.Y)

        self.search_tit = tk.Label(f3, text="Search")
        self.search_tit.pack(side=tk.LEFT, expand=0)
        self.search_var = tk.StringVar()
        self.search_var.trace("w", lambda name, index, mode: self.update_list(all_vars))
        self.entry = tk.Entry(f3, textvariable=self.search_var, width=30)
        self.entry.pack(side=tk.LEFT, expand=0)
        self.lstbox = tk.Listbox(f1, yscrollcommand=right_scrollbar.set, width=80)
        self.lstbox.pack(side=tk.LEFT, fill=tk.BOTH, expand=1)
        right_scrollbar.config(command=self.lstbox.yview)

        self.lstbox.bind('<<ListboxSelect>>', self.on_click)
        self.update_list(all_vars)

        fig = Figure(figsize=(200, 100))
        plt.rc('legend', fontsize='medium')
        self.subplot = fig.add_subplot(111)

        self.canvas = FigureCanvasTkAgg(fig, master=f2)

        self.toolbar = NavigationToolbar2Tk(self.canvas, f2)
        self.canvas.get_tk_widget().pack()

        self.button = Button(self.toolbar, text="Clear data", command=self.clear_plots)
        self.button.pack(side=tk.LEFT, fill=tk.BOTH, expand=0)

        self.toolbar.pack(side=tk.TOP, fill=tk.BOTH, expand=0)
        self.toolbar.update()
        self.canvas.draw()

    def update_list(self, list_):
        search_term = self.search_var.get()

        # populate the listbox with default values
        lbox_list = list_

        self.lstbox.delete(0, tk.END)

        for item in lbox_list:
            if search_term.lower() in item.lower():
                self.lstbox.insert(tk.END, item)

    def clear_plots(self):
        """Clear plot displayed and imported data"""
        self.subplot.clear()
        self.canvas.draw()
        self.py_dict = {}
        self.py_dict2 = {}
        self.time_updated = {}
        self.time_updated2 = {}
        self.scenario = ''
        self.scenario2 = ''

    @staticmethod
    def replace_let_by_x(stri):
        for let in '\"\'-+\%?&\/=\(\)': stri = stri.replace(let, 'x')
        return stri
        
    def open_file(self):
        ext = (("csv", "*.*csv"), ("all files", "*.*"))
        filename = askopenfilename(initialdir="./", title="Open file", filetypes=ext)

        if filename.split('.')[-1] == "csv":
            print("Reading csv file {}".format(filename))
            df = pd.read_csv(filename, index_col=0).T
            df.index = pd.to_numeric(df.index)
            pattern = re.compile(r'results_(.*)(?=_[\d]{4}_[\d]{4}_[\d.]*(_old)*.csv)', re.I)
            try:
                scen_name = pattern.match(os.path.basename(filename)).group(1)
            except:
                print("To be able to import the scenario name, the output file name should be the default one (e.g. results_ScenName_InitDate_FinalDate_TimeStep.csv)")
                scen_name = 'Unknown Scenario'

            if self.py_dict:
                self.py_dict2 = df.to_dict(orient='list')
                self.time_updated2 = df.index.values
                self.scenario2 = scen_name.capitalize()
                print('Data loaded for scenario {}'.format(self.scenario2))
            else:
                self.py_dict = df.to_dict(orient='list')
                self.time_updated = df.index.values
                self.scenario = scen_name.capitalize()
                print('Data loaded for scenario {}'.format(self.scenario))

        else:
            print("Incompatible file format. Compatible file formats are 'csv' "
                  "and... 'csv' (for now)")

    dict_correct = {
        'year': 'Year',
        'ej': 'EJ',
        'tw': 'TW',
        'mdollar': 'M Dollar',
        'pj':'PJ', 'gj':'GJ',
        'beTWeen':'between',
        'DegreesC': u'Temperature (\u00B0C)',
        'degrees Celsius':u'\u00B0C',
        'degree C': u'\u00B0C',
        'degrees C': u'\u00B0C',
        'yr': 'Year',
        'Dmnl': 'Dimensionless'
        }

    def define_units(self, varname):
        def adjust_format(name):
            for wrong in self.dict_correct:
                name = name.replace(wrong, self.dict_correct[wrong])
            return name
        dicti = {'ej': 'EJ', 'mtoe': 'mtoe', 'tw': 'TW', 'twh': 'TWh', 'mt': 'Mt', 'DegreesC': u'Temperature (\u00B0C)'}
        clean_name = varname.split('[')[0].strip(' x')
        ret = None
        for unit in dicti:
            if clean_name.endswith(unit):
                ret = dicti[unit]
                break
        else:
            for name in self.legend_by_name:
                if varname.startswith(name):
                    ret = adjust_format(self.legend_by_name[name][0])
        return ret

    def define_title(self, varname):
        # specific update of the legend
        def adjust_format(name):
            for wrong in self.dict_correct:
                name = name.replace(wrong, self.dict_correct[wrong])
            name = name.replace('\\', '\n').replace('\t', ' ')
            return name.split('.')[0].strip()
        dicti = {}
        clean_name = varname.split('[')[0].strip(' x')
        ret = ''
        for unit in dicti:
            if clean_name.startswith(unit):
                ret = dicti[unit]
                break
        else:
            for name in self.legend_by_name:
                if varname.startswith(name):
                    ret = adjust_format(self.legend_by_name[name][1].split('[')[0]) #.split('(')[0])
        return ret

    def on_click(self, event):

        w = event.widget
        try:
            index = w.curselection()[0]
        except:
            self.column = Plot_tool.last_col
        else:
            self.column = w.get(index)
            Plot_tool.last_col = self.column

        self.draw()

    def draw(self):

        self.subplot.clear()
        self.canvas.draw()
        
        # image name will be named after the plotted variable name
        self.canvas.get_default_filename = lambda: self.column 

        title = self.define_title(self.column)
        title = title + '\n({})'.format(self.column) if title else self.column
        self.subplot.set_title(title)
        self.subplot.set_xlabel('Year')

        ylabel = self.define_units(self.column)

        if ylabel:
            self.subplot.set_ylabel(ylabel, rotation='vertical')

        if self.py_dict:

            if not self.column.lower().startswith('-'):
                print('Plotting ' + self.column)
                nt = len(np.where(self.time_updated < 2014)[0])

                if self.py_dict2:
                    self.subplot.plot(self.time_updated[0:nt], self.py_dict[self.column][0:nt], 'o:', markersize=4, label='Historical ' + self.scenario)
                    self.subplot.plot(self.time_updated2[0:nt], self.py_dict2[self.column][0:nt], '^:', markersize=4, label='Historical ' + self.scenario2)

                    self.subplot.plot(self.time_updated[nt - 1:], self.py_dict[self.column][nt - 1:], 'o-', markersize=4, label='Projected ' + self.scenario)
                    self.subplot.plot(self.time_updated2[nt - 1:], self.py_dict2[self.column][nt - 1:], '^-', markersize=4, label='Projected ' + self.scenario2)
                else:
                    self.subplot.plot(self.time_updated[0:nt], self.py_dict[self.column][0:nt], 'o:', markersize=4, label='Historical ' + self.scenario)
                    self.subplot.plot(self.time_updated[nt - 1:], self.py_dict[self.column][nt - 1:], 'o-', markersize=4, label='Projected ' + self.scenario)

                self.subplot.legend()
                self.canvas.draw()
            else:
                print('Nothing to plot')
        else:
            print('No data available to plot, please load data from a csv file')



def on_closing(root):
    if messagebox.askokcancel("Quit", "Do you want to quit?"):
        root.destroy()
        sys.exit()


def main(folder, df, scenario=''):

    root = tk.Tk()
    root.geometry("1200x600")
    app = Plot_tool(root, data=df, results_folder=folder, scenario=scenario)
    root.protocol("WM_DELETE_WINDOW", lambda: on_closing(root))
    root.mainloop()


if __name__ == '__main__':

    # load configuration file
    conf = ConfigParser()
    conf.read('config.ini')

    region = conf.get('inputs', 'MODEL')
    path_to_results = os.path.join(os.path.dirname(os.path.abspath(__file__)), region)

    main(path_to_results, None)

